package caller;

import api.RestrictedApi;
import org.kohsuke.accmod.restrictions.suppressions.SuppressRestrictedWarnings;

@SuppressRestrictedWarnings(RestrictedApi.class)
public class RestrictedApiSubclass extends RestrictedApi {
}
