package caller;

import api.RestrictedInterface;
import org.kohsuke.accmod.restrictions.suppressions.SuppressRestrictedWarnings;

@SuppressRestrictedWarnings(RestrictedInterface.class)
public class RestrictedInterfaceImplementation implements RestrictedInterface {
}
